package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.openxma.dsl.reference.dao.BugDao;
import org.openxma.dsl.reference.model.Bug;
import org.openxma.dsl.reference.model.impl.BugImpl;
/**
 * @see BugDao
 */
@SuppressWarnings("unused")
public abstract class BugDaoGenImpl extends GenericDaoHibernateImpl<Bug,Long> implements BugDao {

    public BugDaoGenImpl() {
        super(BugImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Bug</code> with an already assigned identifier.
     */
    public Bug createEntity(Object context) {
        return create();
    }

    public Bug create()
    {
        Bug bug = createEntityInstance();
        bug.setOid(Long.valueOf(dataFieldMaxValueIncrementer.nextLongValue()));
        return bug;
    }

    public Bug create(String bugId) {
        Assert.notNull(bugId, "Parameter 'bugId' must not be null");
        Bug bug = (Bug) create();
        bug.setBugId(bugId);
        return bug;
    }

    public Bug createAndSave(String bugId) {
        Bug bug = create(bugId);
        saveOrUpdate(bug);
        return bug;
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new BugDaoQueryMapper();
	}
	
	protected class BugDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
