package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.Set;
import org.openxma.dsl.reference.base.CreditCardInfo;
import org.openxma.dsl.reference.dao.PrivateCustomerDao;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.PrivateCustomer;
import org.openxma.dsl.reference.model.impl.PrivateCustomerImpl;
import org.openxma.dsl.reference.types.Gender;
/**
 * @see PrivateCustomerDao
 */
@SuppressWarnings("unused")
public abstract class PrivateCustomerDaoGenImpl extends GenericDaoHibernateImpl<PrivateCustomer,String> implements PrivateCustomerDao {

    public PrivateCustomerDaoGenImpl() {
        super(PrivateCustomerImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>PrivateCustomer</code> with an already assigned identifier.
     */
    public PrivateCustomer createEntity(Object context) {
        return create();
    }

    public PrivateCustomer create()
    {
        PrivateCustomer privateCustomer = createEntityInstance();
        privateCustomer.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return privateCustomer;
    }

    public PrivateCustomer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo) {
        Assert.notNull(invoiceAddress, "Parameter 'invoiceAddress' must not be null");
        Assert.notNull(deliveryAddress, "Parameter 'deliveryAddress' must not be null");
        Assert.notNull(customerFirstName, "Parameter 'customerFirstName' must not be null");
        Assert.notNull(customerLastName, "Parameter 'customerLastName' must not be null");
        Assert.notNull(gender, "Parameter 'gender' must not be null");
        Assert.notNull(emailAddress, "Parameter 'emailAddress' must not be null");
        Assert.notNull(customerBirthDate, "Parameter 'customerBirthDate' must not be null");
        Assert.notNull(ccInfo, "Parameter 'ccInfo' must not be null");
        PrivateCustomer privateCustomer = (PrivateCustomer) create();
        privateCustomer.setInvoiceAddress(invoiceAddress);
        privateCustomer.setDeliveryAddress(deliveryAddress);
        privateCustomer.setFirstName(customerFirstName);
        privateCustomer.setLastName(customerLastName);
        privateCustomer.setGender(gender);
        privateCustomer.setEmailAddress(emailAddress);
        privateCustomer.setBirthDate(customerBirthDate);
        privateCustomer.setCcInfo(ccInfo);
        return privateCustomer;
    }

    public PrivateCustomer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo) {
        PrivateCustomer privateCustomer = create(invoiceAddress,deliveryAddress,customerFirstName,customerLastName,gender,emailAddress,customerBirthDate,ccInfo);
        saveOrUpdate(privateCustomer);
        return privateCustomer;
    }
    
    public PrivateCustomer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate) {
        Criterion criterion = equal("firstName", customerFirstName, false);
        criterion = and(criterion,"lastName", customerLastName, false);
        criterion = and(criterion,"birthDate", customerBirthDate, false);
        Criteria criteria = createCriteria(criterion);
        return (PrivateCustomer) unique(criteria);
    }

    public Set<PrivateCustomer> findByNameFilter(String customerFirstName,String customerLastName,Date customerBirthDate,Integer firstResult,Integer maxResults) {
        Criterion criterion = equal("firstName", customerFirstName, false);
        criterion = and(criterion,"lastName", customerLastName, false);
        criterion = and(criterion,"birthDate", customerBirthDate, false);
        Criteria criteria = createCriteria(criterion);
        applyPagingParameters(criteria,firstResult,maxResults);
        return new LinkedHashSet<PrivateCustomer>(list(criteria));
    }
        
    public Collection<PrivateCustomer> findAllByFirstNameAndLastName(String firstName,String lastName) {
		String queryName = PrivateCustomerDao.FIND_ALL_BY_FIRST_NAME_AND_LAST_NAME;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("firstName", firstName);
        namedQuery.setParameter("lastName", lastName);
        applyFindAllByFirstNameAndLastNameQueryHints(namedQuery,firstName,lastName);
        return list(namedQuery);
    }

    protected void applyFindAllByFirstNameAndLastNameQueryHints(Query query,String firstName,String lastName) {
        // override this method and apply other hints that influence how a query is executed
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new PrivateCustomerDaoQueryMapper();
	}
	
	protected class PrivateCustomerDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
