package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import org.openxma.dsl.reference.SupplierAddress;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.SupplierNameView;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.model.impl.SupplierImpl;
/**
 * @see SupplierDao
 */
@SuppressWarnings("unused")
public abstract class SupplierDaoGenImpl extends GenericDaoHibernateImpl<Supplier,String> implements SupplierDao {

    public SupplierDaoGenImpl() {
        super(SupplierImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Supplier</code> with an already assigned identifier.
     */
    public Supplier createEntity(Object context) {
        return create();
    }

    public Supplier create()
    {
        Supplier supplier = createEntityInstance();
        supplier.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return supplier;
    }

    public Supplier create(String companyName) {
        Assert.notNull(companyName, "Parameter 'companyName' must not be null");
        Supplier supplier = (Supplier) create();
        supplier.setCompanyName(companyName);
        return supplier;
    }

    public Supplier createAndSave(String companyName) {
        Supplier supplier = create(companyName);
        saveOrUpdate(supplier);
        return supplier;
    }
    
    public Supplier loadBySupplierKey(String companyName) {
        Criterion criterion = equal("companyName", companyName, false);
        Criteria criteria = createCriteria(criterion);
        return (Supplier) unique(criteria);
    }
        
    public Collection<SupplierNameView> findAllByCompanyName(String companyName) {
		String queryName = SupplierDao.FIND_ALL_BY_COMPANY_NAME;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("companyName", companyName);
        applyFindAllByCompanyNameQueryHints(namedQuery,companyName);
        return list(namedQuery);
    }

    protected void applyFindAllByCompanyNameQueryHints(Query query,String companyName) {
        // override this method and apply other hints that influence how a query is executed
    }
        
    public Collection<SupplierNameView> findAllByCompanyNamePaging(String companyName,Integer firstResult,Integer maxResults) {
		String queryName = SupplierDao.FIND_ALL_BY_COMPANY_NAME_PAGING;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("companyName", companyName);
        applyFindAllByCompanyNamePagingQueryHints(namedQuery,companyName,firstResult,maxResults);
        return list(namedQuery);
    }

    protected void applyFindAllByCompanyNamePagingQueryHints(Query query,String companyName,Integer firstResult,Integer maxResults) {
        if (firstResult!=null) {
            query.setFirstResult(firstResult);
        }
        if (maxResults!=null) {
            query.setMaxResults(maxResults);
        }
        // override this method and apply other hints that influence how a query is executed
    }
        
    public Long getSupplierCount() {
		String queryName = SupplierDao.GET_SUPPLIER_COUNT;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        applyGetSupplierCountQueryHints(namedQuery);
        return unique(namedQuery);
    }

    protected void applyGetSupplierCountQueryHints(Query query) {
        // override this method and apply other hints that influence how a query is executed
    }
        
    public Collection<Supplier> findAllByAddress(SupplierAddress address) {
		String queryName = SupplierDao.FIND_ALL_BY_ADDRESS;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("address_streetName", address.getStreetName());
        applyFindAllByAddressQueryHints(namedQuery,address);
        return list(namedQuery);
    }

    protected void applyFindAllByAddressQueryHints(Query query,SupplierAddress address) {
        // override this method and apply other hints that influence how a query is executed
    }
        
    public Collection<Supplier> findWithUnusedParameter(String companyName,String code) {
		String queryName = SupplierDao.FIND_WITH_UNUSED_PARAMETER;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("companyName", companyName);
        applyFindWithUnusedParameterQueryHints(namedQuery,companyName,code);
        return list(namedQuery);
    }

    protected void applyFindWithUnusedParameterQueryHints(Query query,String companyName,String code) {
        // override this method and apply other hints that influence how a query is executed
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new SupplierDaoQueryMapper();
	}
	
	protected class SupplierDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
