package org.openxma.dsl.reference.dtoassembler;

import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.dsl.platform.assembler.DtoAssembler;
import org.openxma.dsl.platform.assembler.DtoAssemblerFactory;
import org.openxma.dsl.platform.assembler.DtoAssemblerRegistry;
import org.openxma.dsl.reference.dtoassembler.dto.ChildView;
import org.openxma.dsl.reference.dtoassembler.dto.ParentView;
import org.openxma.dsl.reference.dtoassembler.model.Child;
import org.openxma.dsl.reference.dtoassembler.model.Parent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;

class DtoAssemblerFactoryDtoassemblerGen implements DtoAssemblerFactory {

	private List<DtoAssembler<?>> dtoAssemblers;

	@Autowired
    private DtoAssemblerRegistry dtoAssemblerRegistry;

    public DtoAssemblerFactoryDtoassemblerGen() {
		dtoAssemblers = new ArrayList<DtoAssembler<?>>(2);
		initDtoAssemblers();
	}


	public List<DtoAssembler<?>> getDtoAssemblers() {
		return dtoAssemblers;
	}

	protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
		return dtoAssemblerRegistry;
	}

	private void initDtoAssemblers() {
		dtoAssemblers.add(createDtoAssemblerChildView());
		dtoAssemblers.add(createDtoAssemblerParentView());
	}


	private DtoAssembler<ChildView> createDtoAssemblerChildView() {
		return new DtoAssembler<ChildView>() {
		    public ChildView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof Child) {
					Child child = (Child) sourceEntity;
		    		ChildView result = mapFromChildToChildView(child);
		    		if (null != child.getParent()) {
		    			result.setParentOid(child.getParent().getOid());
		    		}
		    		return result;
		    	} else if (sourceEntity instanceof Parent) {
					Parent parent = (Parent) sourceEntity;
		    		ChildView result = mapFromParentToChildView(parent);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public ChildView mapFromEntity(Object sourceEntity, ChildView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof Child) {
					Child child = (Child) sourceEntity;
		    		mapFromChildToChildView(child, targetDto);
		    		if (null != child.getParent()) {
		    			targetDto.setParentOid(child.getParent().getOid());
		    		}
		    		return targetDto;
		    	} else if (sourceEntity instanceof Parent) {
					Parent parent = (Parent) sourceEntity;
		    		mapFromParentToChildView(parent, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(ChildView sourceDto, Class<T> target) {
				if (ChildView.class.equals(target)) {
					ChildView result = new ChildView();
					mapFromChildViewToChildView(sourceDto, result);
					return target.cast(result);
				}
				else if (Child.class.equals(target)) {
		    		return target.cast(mapFromChildViewToChild(sourceDto));
		    	}
				else if (Parent.class.equals(target)) {
		    		return target.cast(mapFromChildViewToParent(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(ChildView sourceDto, Object targetObject) {
		    	if (targetObject instanceof ChildView) {
					mapFromChildViewToChildView(sourceDto, (ChildView) targetObject);
				}
		    	else if (targetObject instanceof Child) {
		    		mapFromChildViewToChild(sourceDto, ((Child)targetObject));
		    	}
		    	else if (targetObject instanceof Parent) {
		    		mapFromChildViewToParent(sourceDto, ((Parent)targetObject));
		    	}
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromChildViewToChildView(ChildView sourceDto, ChildView childView) {
   		childView.setParent(sourceDto.getParent());
   		childView.setOid(sourceDto.getOid());
   		childView.setVersion(sourceDto.getVersion());
   		childView.setFirstName(sourceDto.getFirstName());
   		childView.setLastName(sourceDto.getLastName());
   		childView.setParentOid(sourceDto.getParentOid());
    }
    
    protected Child mapFromChildViewToChild(ChildView sourceDto) {
    	return mapFromChildViewToChild(sourceDto, MapperContextHolder.createEntity(Child.class));
    }

    protected Child mapFromChildViewToChild(ChildView sourceDto, Child child) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return child;
    	}
	    MapperContextHolder.getContext().put(sourceDto, child);
		if (sourceDto.getOid() != null) {
			child.setOid(sourceDto.getOid());
		}
		child.setVersion(sourceDto.getVersion());
		child.setFirstName(sourceDto.getFirstName());
		child.setLastName(sourceDto.getLastName());
	    if (!MapperContextHolder.isContext("saveChild") && !MapperContextHolder.isContext("updateChild")) {
			if (null != sourceDto.getParent()) {
		   		Parent parent = (Parent) dtoAssemblerRegistry.getDtoAssembler(ParentView.class).mapTo(sourceDto.getParent(), Parent.class);
				child.setParent(parent);
			}
	    }
	    return child;
	}
			
    protected Parent mapFromChildViewToParent(ChildView sourceDto) {
    	return mapFromChildViewToParent(sourceDto, MapperContextHolder.createEntity(Parent.class));
    }

    protected Parent mapFromChildViewToParent(ChildView sourceDto, Parent parent) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return parent;
    	}
	    MapperContextHolder.getContext().put(sourceDto, parent);
		if (sourceDto.getParentOid() != null  && MapperContextHolder.isContext("parent")) {
			parent.setOid(sourceDto.getParentOid());
		}
	    return parent;
	}
	
	protected ChildView mapFromChildToChildView(Child child) {
		if (MapperContextHolder.getContext().containsKey(child)) {
			return (ChildView) MapperContextHolder.getContext().get(child);
		}
		ChildView childView = new ChildView();
    	MapperContextHolder.getContext().put(child, childView);
		childView.setOid(child.getOid());
		childView.setVersion(child.getVersion());
		childView.setFirstName(child.getFirstName());
		childView.setLastName(child.getLastName());
        if (null != child.getParent()) {
	        ParentView parent = (ParentView) MapperContextHolder.getContext().get(child.getParent());
	        if (null == parent) {
	        	parent = dtoAssemblerRegistry.getDtoAssembler(ParentView.class).mapFromEntity(child.getParent());
	        }
        	childView.setParent(parent);
        }
        return childView;
	}
	
	protected ChildView mapFromChildToChildView(Child child, ChildView childView) {
		if (MapperContextHolder.getContext().containsKey(child)) {
			return childView;
		}		
    	MapperContextHolder.getContext().put(child, childView);
		childView.setOid(child.getOid());
		childView.setVersion(child.getVersion());
		childView.setFirstName(child.getFirstName());
		childView.setLastName(child.getLastName());
        if (null != child.getParent()) {
	        ParentView parent = (ParentView) MapperContextHolder.getContext().get(child.getParent());
	        if (null == parent) {
	        	parent = dtoAssemblerRegistry.getDtoAssembler(ParentView.class).mapFromEntity(child.getParent());
	        }
        	childView.setParent(parent);
        }
        return childView;
	}
	
	protected ChildView mapFromParentToChildView(Parent parent) {
		if (MapperContextHolder.getContext().containsKey(parent)) {
			return (ChildView) MapperContextHolder.getContext().get(parent);
		}
		ChildView childView = new ChildView();
    	MapperContextHolder.getContext().put(parent, childView);
        return childView;
	}
	
	protected ChildView mapFromParentToChildView(Parent parent, ChildView childView) {
		if (MapperContextHolder.getContext().containsKey(parent)) {
			return childView;
		}		
    	MapperContextHolder.getContext().put(parent, childView);
        return childView;
	}

	private DtoAssembler<ParentView> createDtoAssemblerParentView() {
		return new DtoAssembler<ParentView>() {
		    public ParentView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof Parent) {
					Parent parent = (Parent) sourceEntity;
		    		ParentView result = mapFromParentToParentView(parent);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public ParentView mapFromEntity(Object sourceEntity, ParentView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof Parent) {
					Parent parent = (Parent) sourceEntity;
		    		mapFromParentToParentView(parent, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(ParentView sourceDto, Class<T> target) {
				if (ParentView.class.equals(target)) {
					ParentView result = new ParentView();
					mapFromParentViewToParentView(sourceDto, result);
					return target.cast(result);
				}
				else if (Parent.class.equals(target)) {
		    		return target.cast(mapFromParentViewToParent(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(ParentView sourceDto, Object targetObject) {
		    	if (targetObject instanceof ParentView) {
					mapFromParentViewToParentView(sourceDto, (ParentView) targetObject);
				}
		    	else if (targetObject instanceof Parent) {
		    		mapFromParentViewToParent(sourceDto, ((Parent)targetObject));
		    	}
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromParentViewToParentView(ParentView sourceDto, ParentView parentView) {
   		parentView.setChildren(sourceDto.getChildren());
   		parentView.setOid(sourceDto.getOid());
   		parentView.setVersion(sourceDto.getVersion());
   		parentView.setFirstName(sourceDto.getFirstName());
   		parentView.setLastName(sourceDto.getLastName());
    }
    
    protected Parent mapFromParentViewToParent(ParentView sourceDto) {
    	return mapFromParentViewToParent(sourceDto, MapperContextHolder.createEntity(Parent.class));
    }

    protected Parent mapFromParentViewToParent(ParentView sourceDto, Parent parent) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return parent;
    	}
	    MapperContextHolder.getContext().put(sourceDto, parent);
		if (sourceDto.getOid() != null) {
			parent.setOid(sourceDto.getOid());
		}
		parent.setVersion(sourceDto.getVersion());
		parent.setFirstName(sourceDto.getFirstName());
		parent.setLastName(sourceDto.getLastName());
	    if (!MapperContextHolder.isContext("saveParent") && !MapperContextHolder.isContext("updateParent")) {
			if (null != sourceDto.getChildren()) {
				for (ChildView childView : sourceDto.getChildren()) {
					Child children = (Child) dtoAssemblerRegistry.getDtoAssembler(ChildView.class).mapTo(childView, Child.class);
					parent.addChildren(children);
				}
			}
	    }
	    return parent;
	}
	
	protected ParentView mapFromParentToParentView(Parent parent) {
		if (MapperContextHolder.getContext().containsKey(parent)) {
			return (ParentView) MapperContextHolder.getContext().get(parent);
		}
		ParentView parentView = new ParentView();
    	MapperContextHolder.getContext().put(parent, parentView);
		parentView.setOid(parent.getOid());
		parentView.setVersion(parent.getVersion());
		parentView.setFirstName(parent.getFirstName());
		parentView.setLastName(parent.getLastName());
		if (parent.hasChildren()) {
			for (Child next : parent.getChildren()) {
	    		ChildView nextChildView = (ChildView) MapperContextHolder.getContext().get(next);
	    		if (null == nextChildView) {
		        	nextChildView = dtoAssemblerRegistry.getDtoAssembler(ChildView.class).mapFromEntity(next);
		        }
	        	parentView.addChildren(nextChildView);
	    	}
		}
        return parentView;
	}
	
	protected ParentView mapFromParentToParentView(Parent parent, ParentView parentView) {
		if (MapperContextHolder.getContext().containsKey(parent)) {
			return parentView;
		}		
    	MapperContextHolder.getContext().put(parent, parentView);
		parentView.setOid(parent.getOid());
		parentView.setVersion(parent.getVersion());
		parentView.setFirstName(parent.getFirstName());
		parentView.setLastName(parent.getLastName());
		if (parent.hasChildren()) {
			for (Child next : parent.getChildren()) {
	    		ChildView nextChildView = (ChildView) MapperContextHolder.getContext().get(next);
	    		if (null == nextChildView) {
		        	nextChildView = dtoAssemblerRegistry.getDtoAssembler(ChildView.class).mapFromEntity(next);
		        }
	        	parentView.addChildren(nextChildView);
	    	}
		}
        return parentView;
	}

}
