package org.openxma.dsl.reference.dtoassembler.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dtoassembler.dao.ChildDao;
import org.openxma.dsl.reference.dtoassembler.dao.ParentDao;
import org.openxma.dsl.reference.dtoassembler.dto.ChildView;
import org.openxma.dsl.reference.dtoassembler.model.Child;
import org.openxma.dsl.reference.dtoassembler.model.Parent;
import org.openxma.dsl.reference.dtoassembler.service.ChildService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

public abstract class ChildServiceGenImpl implements ChildService {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected ChildDao childDao;
	protected ParentDao parentDao;
	
	@Autowired
	public void setChildDao(ChildDao childDao) {
	    this.childDao = childDao;
	}

	@Autowired
	public void setParentDao(ParentDao parentDao) {
	    this.parentDao = parentDao;
	}
	
	@Transactional
	public ChildView save(ChildView childView) {
		Assert.notNull(childView, "parameter 'childView' must not be null");
        Child child = this.mapper.createAndMapOne(childView, Child.class,"saveChild");
		Parent parent = this.mapper.createAndMapOne(childView, Parent.class,"parent");
		parent = parentDao.load(parent.getOid());
		child.setParent(parent);
        this.childDao.saveOrUpdate(child);
        return this.mapper.mapOne(child,new ChildView());
	}
	
	@Transactional(readOnly = true)
	public ChildView loadChildView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.childDao.load(oid),ChildView.class);
	}
	
	@Transactional
	public void update(ChildView childView) {
		Assert.notNull(childView, "parameter 'childView' must not be null");
		Child mappedChild = this.mapper.createAndMapOne(childView, Child.class);
        Child child = this.childDao.load(mappedChild.getOid());
        if (!child.getVersion().equals(mappedChild.getVersion())) {
        	throw new OptimisticLockingFailureException("Entity 'Child' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(childView, child,"updateChild");
	}
	
	@Transactional
	public void deleteChild(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        this.childDao.delete(oid);
	}

}
