package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.base.model.impl.BaseEntityImpl;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.types.Gender;
import org.openxma.dsl.reference.types.valueobject.TelefonNumber;

public abstract class CustomerGenImpl extends BaseEntityImpl implements Customer {
	protected String firstName;
	protected String lastName;
	protected Gender gender;
	protected String emailAddress;
	protected Date birthDate;
	protected TelefonNumber telefonNumber;
	protected Address invoiceAddress;
	protected Address deliveryAddress;
	protected Set<Order> orders;
    
	public String getFirstName() {
		return this.firstName;
	}
	
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return this.lastName;
	}
	
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public Gender getGender() {
		return this.gender;
	}
	
	public void setGender(Gender gender) {
		this.gender = gender;
	}

	public String getEmailAddress() {
		return this.emailAddress;
	}
	
	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public Date getBirthDate() {
		return this.birthDate;
	}
	
	public void setBirthDate(Date birthDate) {
		this.birthDate = birthDate;
	}

	public TelefonNumber getTelefonNumber() {
		return this.telefonNumber;
	}
	
	public void setTelefonNumber(TelefonNumber telefonNumber) {
		this.telefonNumber = telefonNumber;
	}

	public Address getInvoiceAddress() {
		return this.invoiceAddress;
	}
	
	public void setInvoiceAddress(Address invoiceAddress) {
		if (this.invoiceAddress != invoiceAddress) {
			this.invoiceAddress = invoiceAddress;
		}
	}

	public Address getDeliveryAddress() {
		return this.deliveryAddress;
	}
	
	public void setDeliveryAddress(Address deliveryAddress) {
		if (this.deliveryAddress != deliveryAddress) {
			this.deliveryAddress = deliveryAddress;
		}
	}

	public Set<Order> getOrders() {
		if (this.orders == null) {
            this.orders = new HashSet<Order>();
        }
        return Collections.unmodifiableSet(this.orders);
	}
	
	public boolean hasOrders() {
		return orders != null && !orders.isEmpty();
	}

	public void addOrders(Order orders) {
		if (orders == null) {
			throw new IllegalArgumentException("parameter 'orders' must not be null");
		}
		if (!getOrders().contains(orders)) {
			this.orders.add(orders);
			orders.setCustomer(this);
		}
	}

	public void removeOrders(Order orders) {
		if (orders == null) {
			throw new IllegalArgumentException("parameter 'orders' must not be null");
		}
		if (getOrders().contains(orders)) {
            this.orders.remove(orders);
			orders.setCustomer(null);
		}
	}

    public void removeAllOrders() {
        if (this.orders != null) {
            this.orders.clear();
        }
    }
	
    @Override
    public String toString() {
    	StringBuilder toStringBuilder = new StringBuilder("Customer [");
    	toStringBuilder.append(super.toString()).append(",");
        toStringBuilder.append("firstName=").append(getFirstName()).append(",");
        toStringBuilder.append("lastName=").append(getLastName()).append(",");
        toStringBuilder.append("gender=").append(getGender()).append(",");
        toStringBuilder.append("emailAddress=").append(getEmailAddress()).append(",");
        toStringBuilder.append("birthDate=").append(getBirthDate()).append(",");
        toStringBuilder.append("telefonNumber=").append(getTelefonNumber()).append(",");
        toStringBuilder.append("lastOrderDate=").append(getLastOrderDate());
        return toStringBuilder.append("]").toString();
    }
}
