package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.VODataDao;
import org.openxma.dsl.reference.dto.VODataView;
import org.openxma.dsl.reference.model.VOData;
import org.openxma.dsl.reference.service.VODataService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

public abstract class VODataServiceGenImpl implements VODataService {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected VODataDao vODataDao;
	
	@Autowired
	public void setVODataDao(VODataDao vODataDao) {
	    this.vODataDao = vODataDao;
	}
	
	@Transactional
	public VODataView save(VODataView vODataView) {
		Assert.notNull(vODataView, "parameter 'vODataView' must not be null");
        VOData vOData = this.mapper.createAndMapOne(vODataView, VOData.class,"saveVOData");
        this.vODataDao.saveOrUpdate(vOData);
        return this.mapper.mapOne(vOData,new VODataView());
	}
	
	@Transactional(readOnly = true)
	public VODataView loadVODataView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.vODataDao.load(oid),VODataView.class);
	}
	
	@Transactional
	public void update(VODataView vODataView) {
		Assert.notNull(vODataView, "parameter 'vODataView' must not be null");
		VOData mappedVOData = this.mapper.createAndMapOne(vODataView, VOData.class);
        VOData vOData = this.vODataDao.load(mappedVOData.getOid());
        if (!vOData.getVersion().equals(mappedVOData.getVersion())) {
        	throw new OptimisticLockingFailureException("Entity 'VOData' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(vODataView, vOData,"updateVOData");
	}
	
	@Transactional
	public void deleteVOData(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        this.vODataDao.delete(oid);
	}

}
