package org.openxma.dsl.reference.validation;

import org.springframework.validation.Errors;
import org.openxma.dsl.reference.base.validation.BaseEntityValidator;
import org.openxma.dsl.reference.model.Order;
import org.springframework.validation.ValidationUtils;

public abstract class OrderGenValidator extends BaseEntityValidator {
    
    public boolean isOrderNumberRequired(Order order) {
        return Boolean.TRUE;
    }
    
    public boolean isStoreIdRequired(Order order) {
        return Boolean.TRUE;
    }
    
    public boolean isPlacementDateRequired(Order order) {
        return Boolean.TRUE;
    }
    
    public boolean isDeliveryDateRequired(Order order) {
        return Boolean.FALSE;
    }
    
    public boolean isOrderStateRequired(Order order) {
        return Boolean.TRUE;
    }
    
    public boolean isTotalAmountRequired(Order order) {
        return Boolean.TRUE;
    }
    
    public boolean isCustomerRequired(Order order) {
        return Boolean.TRUE;
    }
        
    @SuppressWarnings("unchecked")
    public boolean supports(Class clazz) {
        return Order.class.isAssignableFrom(clazz);
    }
    
    public Errors validate(Order order) {
        Errors errors = createErrors(order);      
        super.validate(order,errors);
        validate(order,errors);
        return errors;
    }

    public void validate(Object object, Errors errors) {
        Order order = (Order) object;
        if (isOrderNumberRequired(order)) {
            ValidationUtils.rejectIfEmptyOrWhitespace(errors, "orderNumber", "field.required");
        }
        if (isStoreIdRequired(order)) {
            ValidationUtils.rejectIfEmptyOrWhitespace(errors, "storeId", "field.required");
        }
        if (isPlacementDateRequired(order)) {
            ValidationUtils.rejectIfEmpty(errors, "placementDate", "field.required");
        }
        if (isDeliveryDateRequired(order)) {
            ValidationUtils.rejectIfEmpty(errors, "deliveryDate", "field.required");
        }
        if (isOrderStateRequired(order)) {
            ValidationUtils.rejectIfEmptyOrWhitespace(errors, "orderState", "field.required");
        }
        if (isTotalAmountRequired(order)) {
            ValidationUtils.rejectIfEmpty(errors, "totalAmount", "field.required");
        }
        if (isCustomerRequired(order)) {
            ValidationUtils.rejectIfEmpty(errors, "customer", "field.required");
        }
        rejectIfMaxLength(errors,"orderState",order.getOrderState(),33);
    }
    
}
