package org.openxma.dsl.reference.xma.order.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.component.*;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;

/**
 * This class implements the generic client part of OrderEditForm.
 * It implements the widgets and models of the page.
 *
 */
public abstract class OrderEditFormGen extends org.openxma.dsl.platform.xma.client.DslDialogPage {
    Notebook tabfolder1;
    TabFolder tabfolder1W;
    Page1 page1;
    Page2 page2;
    Composite actionCompositeW;
    Composite actionComposite_set1W;
    Label actionComposite_set1_centered;
    Button okButtonW;
    Composite actionComposite_set0W;
    Label actionComposite_set0_centered;
    Button cancelButtonW;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the OrderEditFormGen.
     * @param parent the parent page
     */
    public OrderEditFormGen(PageClient parent) {
        super(parent,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the OrderEditFormGen.
     * @param component the component this page belongs to
     */
    public OrderEditFormGen(ComponentClient component) {
        super(component,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the OrderEditFormGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     */
    public OrderEditFormGen(ComponentClient component, Shell parentShell) {
        super(component,parentShell,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the OrderEditFormGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     * @param modal the SWT-style for the desired modality
     */
    public OrderEditFormGen(ComponentClient component, Shell parentShell, int modal) {
        super(component,parentShell,false,modal|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }


    /**
     * Gives the URI of the help for this page
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this page
     */
    public String getHelpUri() {
        return "help/org/openxma/dsl/reference/xma/order/OrderEditForm.html";
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        Locale locale = getComponent().getContext().getLocale();

        widgetModels = new WModel[] {  };
        tabfolder1 = new Notebook(this);
        page1 = new Page1(tabfolder1);
        tabfolder1.addPage(page1);
        page2 = new Page2(tabfolder1);
        tabfolder1.addPage(page2);
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Order.OrderEditFormClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Order getTypedComponent() {
        return (Order) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        FormData data;
        Control[] tabOrder;
        EventAdapter adapter = new EventAdapter(this);
        Locale locale = getComponent().getContext().getLocale();
        ResourceBundle messages = ResourceBundle.getBundle("org.openxma.dsl.reference.xma.order.client.Order",locale);
        final Scaler scaler = Scaler.getInstance(getComposite());
        Control lastControl;

        final Shell shell = getShell();
        if(getDialog()==this) {
            shell.setText(messages.getString("OrderEditForm"));
            shell.addHelpListener(adapter);
        } else {
            getComposite().addHelpListener(adapter);
        }
        Composite orderEditFormCompW = getComposite();
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        orderEditFormCompW.setLayout(layout);

        tabfolder1W = (TabFolder)tabfolder1.createComposite(orderEditFormCompW);
        tabfolder1W.addHelpListener(tabfolder1);
        TabItem page1W = page1.createTabItem(tabfolder1W,0);
        TabItem page2W = page2.createTabItem(tabfolder1W,1);
        tabfolder1W.setData("WIDGET_ID","tabfolder1W");

        actionCompositeW = new Composite(orderEditFormCompW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        actionCompositeW.setLayout(layout);
        actionCompositeW.setData("WIDGET_ID","actionCompositeW");

        actionComposite_set1W = new Composite(actionCompositeW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(0);
        layout.marginWidth = scaler.convertXToCurrent(0);
        actionComposite_set1W.setLayout(layout);
        actionComposite_set1W.setData("WIDGET_ID","actionComposite_set1W");

        actionComposite_set1_centered = new Label (actionComposite_set1W,SWT.LEFT);
        actionComposite_set1_centered.setVisible(false);
        actionComposite_set1_centered.setData("WIDGET_ID","actionComposite_set1_centered");

        okButtonW = new Button (actionComposite_set1W, SWT.PUSH|SWT.CENTER);
        okButtonW.setText(messages.getString("OrderEditForm.okButtonW"));
        okButtonW.setData("WIDGET_ID","okButtonW");
        okButtonW.addSelectionListener(adapter);

        actionComposite_set0W = new Composite(actionCompositeW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(0);
        layout.marginWidth = scaler.convertXToCurrent(0);
        actionComposite_set0W.setLayout(layout);
        actionComposite_set0W.setData("WIDGET_ID","actionComposite_set0W");

        actionComposite_set0_centered = new Label (actionComposite_set0W,SWT.LEFT);
        actionComposite_set0_centered.setVisible(false);
        actionComposite_set0_centered.setData("WIDGET_ID","actionComposite_set0_centered");

        cancelButtonW = new Button (actionComposite_set0W, SWT.PUSH|SWT.CENTER);
        cancelButtonW.setText(messages.getString("OrderEditForm.cancelButtonW"));
        cancelButtonW.setData("WIDGET_ID","cancelButtonW");
        cancelButtonW.addSelectionListener(adapter);

        widgets = new Control[] { tabfolder1W, actionCompositeW, actionComposite_set1W, actionComposite_set1_centered, okButtonW, actionComposite_set0W, actionComposite_set0_centered, cancelButtonW };

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(actionCompositeW,scaler.convertYToCurrent(0),SWT.TOP);
        tabfolder1W.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        actionCompositeW.setLayoutData(data);

        lastControl=null;
        data = new FormData();
            data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        actionComposite_set0W.setLayoutData(data);
        lastControl=actionComposite_set0W;

        data = new FormData();
        if(lastControl!=null) {
            data.right = new FormAttachment(lastControl,scaler.convertXToCurrent(-3),SWT.LEFT);
        } else {
            data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        }
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        actionComposite_set1W.setLayoutData(data);
        lastControl=actionComposite_set1W;

        data = new FormData();
        data.width = scaler.convertXToCurrent(1);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        actionComposite_set0_centered.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(actionComposite_set0_centered,scaler.convertYToCurrent(0),SWT.CENTER);
        cancelButtonW.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(1);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        actionComposite_set1_centered.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(actionComposite_set1_centered,scaler.convertYToCurrent(0),SWT.CENTER);
        okButtonW.setLayoutData(data);


        if(getDialog()==this) {
            shell.setTabList(new Control[] { orderEditFormCompW });
        }
        tabOrder = new Control[2];
        tabOrder[0] = tabfolder1W;
        tabOrder[1] = actionCompositeW;
        orderEditFormCompW.setTabList(tabOrder);
        tabOrder = new Control[2];
        tabOrder[0] = actionComposite_set1W;
        tabOrder[1] = actionComposite_set0W;
        actionCompositeW.setTabList(tabOrder);
        tabOrder = new Control[1];
        tabOrder[0] = okButtonW;
        actionComposite_set1W.setTabList(tabOrder);
        tabOrder = new Control[1];
        tabOrder[0] = cancelButtonW;
        actionComposite_set0W.setTabList(tabOrder);
        if(getDialog()==this) {
            shell.setSize(scaler.convertXToCurrent(640), scaler.convertYToCurrent(480));
        }
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        tabfolder1W = null;
        actionCompositeW = null;
        actionComposite_set1W = null;
        actionComposite_set1_centered = null;
        okButtonW = null;
        actionComposite_set0W = null;
        actionComposite_set0_centered = null;
        cancelButtonW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
    }

    /**
     * Method called to start the DialogPage.
     */
    public boolean invoke() {
        readOrder(null);
        return super.invoke();
    }

    /**
     * This method will be called every time the OrderEditForm becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == okButtonW) {
            saveOrder(event);
        } else if (event.widget == cancelButtonW) {
            cancel(event);
        }
    }



    protected void readOrder(SelectionEvent event) {
        newRemoteCall("readOrder").execute();
    }

    protected void saveOrder(SelectionEvent event) {
        newRemoteCall("saveOrder").execute();
    }

    protected void cancel(SelectionEvent event) {
        newRemoteCall("cancel").execute();
    }

    protected void selectOrderItem(SelectionEvent event) {
        newRemoteCall("selectOrderItem").execute();
    }

}
